@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
+import datetime, random
 
-import datetime
 from django.db import models
 from django.db.models import Count, Sum, Max, Min
 from django.db.models.functions import TruncMonth, TruncDay
@@ -11,6 +11,7 @@ from django.apps import apps
 from django.utils import timezone
 
 from .dt import get_current_week_range_datetime, get_now, LOCAL_TIMEZONE
+from .signals import add_prize_coin
 
 
 # Create your models here.
@@ -81,6 +82,8 @@ class Stage(models.Model):
     level = models.IntegerField(verbose_name='阶段', null=True)
     name = models.CharField(max_length=180, verbose_name='标题')
     created_at = models.DateTimeField(auto_now_add=True)
+    prize_min = models.IntegerField(default=0, verbose_name="奖励m币下限")  # 奖励m币下限
+    prize_max = models.IntegerField(default=0, verbose_name='奖励m币上限')  # 奖励m币上限
 
     class Meta:
         unique_together = ("line", "level")
@@ -105,10 +108,50 @@ class Stage(models.Model):
         except:
             return
 
+    # 获取该stage完成后的奖励
+    def get_prize_coin(self):
+        prize_list = StagePrize.objects.filter(stage=self)
+        _random = random.random()
+        _sum = 0
+        for prize in prize_list:
+            _sum += prize.chance
+            if _sum > 1 :
+                return self.get_min_prize_coin()
+            if  _random <= _sum:
+                return prize.coin
+        return self.get_min_prize_coin()
+
+    def get_max_prize_coin(self):
+        data_list = StagePrize.objects.filter(stage=self).order_by("-coin")
+        if data_list.exists():
+            return data_list.first().coin
+        return 0
+
+    def get_min_prize_coin(self):
+        data_list = StagePrize.objects.filter(stage=self).order_by("coin")
+        if data_list.exists():
+            return data_list.first().coin
+        return 0
+
+
+class StagePrize(models.Model):
+    stage = models.ForeignKey(Stage, null=True, verbose_name='所属阶段')
+    coin = models.IntegerField(verbose_name='奖励M币')
+    chance = models.FloatField(verbose_name='获得几率')
+
+    class Meta:
+        db_table = 'bee_django_mission_stage_prize'
+        app_label = 'bee_django_mission'
+        ordering = ['pk']
+
+    def __unicode__(self):
+        return self.stage.name+":coin-"+self.coin.__str__()+",chance-"+self.chance.__str__()
+
 
 MISSION_AGGREGATE_TYPE_CHOICES = ((1, "Count"), (2, "Sum"), (3, "TruncDay"))
 MISSION_COMPARISON_TYPE_CHOICES = ((1, '>='), (2, '>'))
 MISSION_OPERATOR_TYPE_CHOICES = ((0, '无'), (1, '* 60'),)
+MISSION_LINK_TYPE_CHOICES = ((0, '无'), (1, 'url后面接user.id'),)
 
 
 class MissionType(models.Model):
@@ -127,6 +170,8 @@ class MissionType(models.Model):
 
     link_url = models.CharField(max_length=180, null=True, blank=True, verbose_name='链接地址')
     link_name = models.CharField(max_length=180, null=True, blank=True, verbose_name='链接名字')
+    line_type = models.IntegerField(verbose_name='对值运算', choices=MISSION_LINK_TYPE_CHOICES,
+                                    default=0)
 
     class Meta:
         db_table = 'bee_django_mission_type'
@@ -203,9 +248,9 @@ class UserLine(models.Model):
 
     # 获取该学生，同一user_line下，正在进行的user_stage，没有返回空
     def get_woking_user_stage(self):
-        woking_user_stage_list = UserStage.objects.filter(user_line=self, stutas=0).order_by(
+        woking_user_stage_list = UserStage.objects.filter(user_line=self, finish_at__isnull=True).order_by(
             'finish_at')
-        if woking_user_stage_list.count() >= 1:
+        if woking_user_stage_list.exists():
             return woking_user_stage_list.last()
         else:
             return None
@@ -215,26 +260,22 @@ class UserLine(models.Model):
     # check：检查是否有未完成的stage
     # 返回新添加的user_stage
     def check_add_user_stage(self, check=True):
-        # 已经有未完成的stage
+
         if check:
-            woking_stage = self.get_woking_user_stage()
-            if woking_stage:
-                return
+            if self.line.is_unlimited_line():
+                # 已经有未完成的stage
+                woking_stage = self.get_woking_user_stage()
+                if woking_stage:
+                    return
+            elif self.line.is_week_line():
+                # 检查是否已有当周任务
+                start_dt, end_dt = get_current_week_range_datetime()
+
+                woking_stage_list = UserStage.objects.filter(user_line=self, start_at=start_dt, end_at=end_dt)
+                if woking_stage_list.exists():
+                    return
         return self.add_user_stage()
 
-    #     return
-
-    # 添加下一个或第一个user_stage
-    # 参数：check检查是否已经有同类型的已开启的user_stage。为true时，不添加，false时，强行再添加一个
-    # def check_add_user_stage(self, check=True):
-    #     # self.update_user_stage_status()
-    #     # 已经有未完成的stage
-    #     if check == True:
-    #         woking_stage = self.get_woking_user_stage()
-    #         if woking_stage:
-    #             return
-    #     self.add_user_stage()
-
     # 添加user_stage 返回新添加的user_stagae
     def add_user_stage(self):
         # 找到第一个,或最后一个user_stage，包括已完成和未完成
@@ -250,13 +291,13 @@ class UserLine(models.Model):
         else:
             next_stage = None
 
-        # 如果该stage下还未添加mission，则不添加
-        # mission_list = next_stage.mission_set.all()
-        # if mission_list.count() == 0:
-        #     return
-
         # 添加阶段任务
         if next_stage:
+            # 如果该stage下还未添加mission，则不添加
+            mission_list = next_stage.mission_set.all()
+            if not mission_list.exists():
+                return
+
             new_user_stage = UserStage()
             new_user_stage.user_line = self
             new_user_stage.stage = next_stage
@@ -266,6 +307,7 @@ class UserLine(models.Model):
                 new_user_stage.start_at = start_dt
                 new_user_stage.end_at = end_dt
                 new_user_stage.name = start_dt.strftime("%Y") + '年第' + start_dt.strftime("%W") + "周任务"
+
             new_user_stage.save()
         else:
             return None
@@ -284,6 +326,7 @@ class UserStage(models.Model):
     finish_at = models.DateTimeField(null=True, verbose_name='完成时间')
     end_at = models.DateTimeField(null=True, verbose_name='结束时间')
     stutas = models.IntegerField(default=0, verbose_name='状态')
+    prize_coin = models.IntegerField(default=0, verbose_name='奖励的m币')
 
     class Meta:
         db_table = 'bee_django_mission_user_stage'
@@ -303,6 +346,7 @@ class UserStage(models.Model):
         self.add_user_mission()
 
     # 添加当前user_stage下的所有mission，如果已添加，则不重复添加
+    # 单个任务如有自定义模版，则按照模版规则添加
     def add_user_mission(self):
         # 所有user_stage下的mission
         mission_list = Mission.objects.filter(stage=self.stage)
@@ -312,10 +356,20 @@ class UserStage(models.Model):
             try:
                 UserMission.objects.get(mission=mission, user_stage=self)
             except:
-                new_user_misssion = UserMission()
-                new_user_misssion.user_stage = self
-                new_user_misssion.mission = mission
-                new_user_misssion.save()
+                pass
+
+            new_user_misssion = UserMission()
+            new_user_misssion.user_stage = self
+            new_user_misssion.mission = mission
+            # 查看学生自定义任务模版，如果有，则更改为自定义任务
+            try:
+                custom_mission = CustomUserMission.objects.get(user=self.user_line.user, stage=self.stage,
+                                                               mission=mission)
+                new_user_misssion.custom_name = custom_mission.custom_name
+                new_user_misssion.custom_count = custom_mission.custom_count
+            except:
+                pass
+            new_user_misssion.save()
         return
 
     # 更新该user_stage下，所有的mission完成情况
@@ -335,8 +389,7 @@ class UserStage(models.Model):
         # 所有mission都已完成
         if unfinish_list.count() == 0:
             if self.user_line.line.auto_finish:
-                self.finish_at = get_now()
-                self.stutas = 1
+                self.finish_user_stage_add_prize()
             else:
 
                 self.stutas = 3
@@ -348,6 +401,77 @@ class UserStage(models.Model):
         self.save()
         return self.stutas
 
+    def finish_user_stage_add_prize(self):
+        self.finish_at = get_now()
+        self.stutas = 1
+        coin = self.stage.get_prize_coin()
+        reason = '完成任务奖励'
+        if self.user_line.line.is_week_line():
+
+            reason = '完成' + self.end_at.strftime("%Y") + '年第' + self.end_at.strftime("%W") + "周任务奖励"
+
+        elif self.user_line.line.is_unlimited_line():
+            reason = '完成' + self.get_name() + "任务奖励"
+        add_prize_coin.send(sender=UserStage, user=self.user_line.user, coin=coin, reason=reason)
+        self.prize_coin = coin
+        self.save()
+        self.update_finish_count()
+
+    # 更新user_mission为最新的stage中的mission
+    def update(self):
+        stage = self.stage
+        user = self.user_line.user
+        mission_list = Mission.objects.filter(stage=stage)
+        for mission in mission_list:
+            # 检查重复，没有则创建
+            try:
+                UserMission.objects.get(mission=mission, user_stage=self)
+                continue
+            except:
+                user_mission = UserMission()
+                user_mission.user = user
+                user_mission.mission = mission
+                user_mission.user_stage = self
+                user_mission.save()
+
+        return
+
+    # 更新用户完成阶段任务的数量，用于排行榜
+    def update_finish_count(self):
+        user = self.user_line.user
+        line = self.user_line.line
+        user_finish_stage_list = UserStage.objects.filter(user_line__user=user,
+                                                          user_line__line__line_type=line.line_type,
+                                                          stutas=1)
+        count = user_finish_stage_list.count()
+        try:
+            user_count = UserStageFinishCount.objects.get(user=user, line=line)
+        except:
+            user_count = UserStageFinishCount()
+            user_count.user = user
+            user_count.line = line
+        user_count.finish_count = count
+        user_count.save()
+        return
+
+
+# 自定义学生任务模版
+class CustomUserMission(models.Model):
+    user = models.ForeignKey(settings.AUTH_USER_MODEL)
+    stage = models.ForeignKey(Stage)
+    mission = models.ForeignKey(Mission)
+    custom_name = models.CharField(max_length=180, blank=True, null=True, verbose_name='自定义名字')
+    custom_count = models.IntegerField(blank=True, null=True, verbose_name='自定义数量')
+
+    class Meta:
+        db_table = 'bee_django_mission_custom_user_mission'
+        app_label = 'bee_django_mission'
+        ordering = ['pk']
+        verbose_name = '学生自定义任务'
+
+    def __str__(self):
+        return self.stage.name
+
 
 class UserMission(models.Model):
     # line = models.ForeignKey(Line)
@@ -363,6 +487,7 @@ class UserMission(models.Model):
         app_label = 'bee_django_mission'
         ordering = ['finish_at']
         verbose_name = '学生的任务'
+        permissions = ('can_reset_user_mission', '可以重置学生的任务'),
 
     def __str__(self):
         return self.mission.name
@@ -376,14 +501,31 @@ class UserMission(models.Model):
         return self.mission.name
 
     def get_count(self):
-        if self.custom_count:
+        if self.custom_count == None:
+            return self.mission.count
+        else:
             return self.custom_count
-        return self.mission.count
+
+    def get_link(self):
+        mission_type = self.mission.mission_type
+        if mission_type.link_url:
+            if mission_type.line_type == 0:
+                return mission_type.link_url
+            if mission_type.line_type == 1:
+                return mission_type.link_url + self.user_stage.user_line.user.id.__str__()
 
     # 获取user_mission的完成情况
+    # 返回【是否完成，已完成，要求，百分比】
     def get_user_mission_progress(self):
-        if self.finish_at:
-            return 1
+        operator_type = self.mission.mission_type.operator_type  # 对count做运算
+        count = self.get_count()
+        # 前台显示用
+        show_require = count
+        if operator_type == 1:
+            count = count * 60
+        # print(count,show_require)
+        # if self.finish_at:
+        #     return True, show_require, show_require, 100
         # 表
         content_type = self.mission.mission_type.content_type
         app_name = content_type.app_label
@@ -393,25 +535,18 @@ class UserMission(models.Model):
         # print(app, model)
         # 查询条件
         user_field = content_type.user_field
-
         aggregate_type = self.mission.mission_type.aggregate_type
         field_name = self.mission.mission_type.field_name
         timestamp_field = self.mission.mission_type.timestamp_field
         comparison_type = self.mission.mission_type.comparison_type  # 大于等于小于
-        operator_type = self.mission.mission_type.operator_type  # 对count做运算
         conditions = self.mission.mission_type.conditions
-        count = self.get_count()
-        if operator_type == 1:
-            count = count * 60
-
-
 
         # 查询
         try:
             queryset = model.objects.all()
-            # print(queryset)
             if queryset.count() == 0:
-                return 0
+                return False, 0, show_require, 0
+            # print(queryset)
             kwargs = {}  # 动态查询的字段
             # name_field = get_user_name_field()
             kwargs[user_field] = self.user_stage.user_line.user
@@ -430,7 +565,7 @@ class UserMission(models.Model):
             # print(kwargs)
             queryset = queryset.filter(**kwargs)
             if queryset.count() == 0:
-                return 0
+                return False, 0, show_require, 0
             # print(queryset)
             # 聚合查询
             if aggregate_type == 1:  # count
@@ -445,7 +580,7 @@ class UserMission(models.Model):
                 queryset["_agg"] = _queryset.count()
 
             else:
-                return 0
+                return False, 0, 0
             # 比较
             if comparison_type == 1:
                 res = queryset["_agg"] >= count
@@ -454,18 +589,41 @@ class UserMission(models.Model):
             else:
                 res = None
             # print(res)
-            if res:
-                # print('finished')
-                self.finish_at = get_now()
-                self.save()
-                return 1
+            if operator_type == 1:
+                show_finish = queryset["_agg"] / 60
             else:
+                show_finish = queryset["_agg"]
+            if show_require == 0:
+                percent = 100
+            else:
+                percent = show_finish * 100 / show_require
+            if self.finish_at:
+                return True, show_finish, show_require, percent
+            else:
+                if res:
+                    # print('finished')
+                    self.finish_at = get_now()
+                    self.save()
+                    return True, show_finish, show_require, percent
+                else:
 
-                # print(queryset["_agg"])
-                # print('not finished')
-                return round(queryset["_agg"].__int__(), 2) / count.__int__()
+                    # print(queryset["_agg"])
+                    # print('not finished')
+                    return False, show_finish, show_require, percent
 
 
         except Exception as e:
             print(e)
-        return 0
+        return False, 0, 0
+
+
+class UserStageFinishCount(models.Model):
+    user = models.ForeignKey(settings.AUTH_USER_MODEL)
+    finish_count = models.IntegerField(default=0)
+    line = models.ForeignKey(Line)
+
+    def __str__(self):
+        return self.user + ":count:" + self.finish_count.__str__()
+
+    def __unicode__(self):
+        return self.user + ":count:" + self.finish_count.__str__()
