@@ -2,12 +2,15 @@
 # -*- coding:utf-8 -*-
 
 __author__ = 'bee'
-
+import datetime
 from django.dispatch import receiver
 from django.db.models.signals import post_save
+from django.utils import timezone
+
 from .models import Stage, UserStage, UserMission, Mission, Line, UserLine
 from .dt import get_current_week_range_datetime
 
+
 # ============receiver============
 # 创建user_line后，自动创建该用户对应的stage
 @receiver(post_save, sender=UserLine)
@@ -33,16 +36,35 @@ def check_add_user_line(user, line_type, check=True):
     if check:
         try:
             UserLine.objects.get(user=user, line__line_type=line_type)
-            return None
+            return None, '该学生还没有任务线，请先去添加'
         except:
             pass
-    line = Line.objects.all().filter(line_type=line_type).first()
-    u_l = UserLine()
-    u_l.line = line
-    u_l.user = user
-    u_l.save()
-    return u_l
 
+    line = Line.objects.all().filter(line_type=line_type).first()
+    res, msg = check_add_line_stage_to_user(line)
+    if res:
+        u_l = UserLine()
+        u_l.line = line
+        u_l.user = user
+        u_l.save()
+        return u_l, None
+    return None, msg
+
+
+# 检查是否可以给学生添加[line或stage]
+# 如果line下没有stage，或stage下没有mission，则不通过
+def check_add_line_stage_to_user(line=None, stage=None):
+    if not line and not stage:
+        return False, '缺少参数'
+    if line:
+        stage = line.get_next_stage()
+        if not stage:
+            return False, '该任务线还没有阶段任务，请先去添加'
+    if stage:
+        mission_list = stage.mission_set.all()
+        if not mission_list.exists():
+            return False, '该阶段任务还没有具体任务，请先去添加'
+    return True, None
 
 
 # 则根据课件是否自动通过，开启下一user_stage
@@ -60,11 +82,11 @@ def check_add_user_line(user, line_type, check=True):
 # 获取学生的当周user_stage
 def get_current_week_stage(user):
     _start_date, _end_date = get_current_week_range_datetime()
-    try:
-        user_stage = UserStage.objects.get(start_at=_start_date, end_at=_end_date, user_line__user=user)
-        return user_stage
-    except:
-        return None
+    user_stage_list = UserStage.objects.filter(start_at=_start_date, end_at=_end_date, user_line__user=user)
+    if user_stage_list.exists():
+        return user_stage_list.first()
+    return None
+
 
 # ============
 # 获取学生的任务线
@@ -86,23 +108,12 @@ def _get_user_line(user, line_type):
 
 # ====================
 
-
-# 更新user_mission为最新的stage中的mission
-
-def change_user_mission(user_stage):
-    stage = user_stage.stage
-    user = user_stage.user_line.user
-    mission_list = Mission.objects.filter(stage=stage)
-    for mission in mission_list:
-        # 检查重复，没有则创建
-        try:
-            UserMission.objects.get(mission=mission, user_stage=user_stage)
-            continue
-        except:
-            user_mission = UserMission()
-            user_mission.user = user
-            user_mission.mission = mission
-            user_mission.save()
+def get_top_prize():
+    now = timezone.now()
+    # 获取15天内
+    start_dt=now + datetime.timedelta(days=-15)
+    prize_list=UserStage.objects.filter(finish_at_gt=start_dt).order_by('-prize_coin')
+    return prize_list
 
 # 给某学生添加周任务，check检查是否已有周任务，如果有，则不添加。默认检查
 # def add_user_week_line(user, check=True):
