@@ -16,11 +16,15 @@ from django.apps import apps
 from django.contrib.auth.models import User
 from django.utils.decorators import method_decorator
 from django import forms
+from django.contrib.auth.decorators import permission_required
+from django.db import transaction
 
 from .decorators import cls_decorator, func_decorator
-from .models import Line, Stage, Mission, UserLine, UserStage, UserMission
-from .utils import check_add_user_line, get_current_week_stage
-from .forms import MissionForm, StageUnlimitedForm, UserLineCreateForm
+from .models import Line, Stage, Mission, UserLine, UserStage, UserMission, UserStageFinishCount, CustomUserMission
+from .utils import check_add_user_line, get_current_week_stage, get_top_prize
+from .forms import MissionForm, StageUnlimitedForm, UserLineCreateForm, UserMissionForm
+from .http import JSONResponse
+from .dt import get_now
 
 
 # from .forms import UserStageFinishForm
@@ -29,6 +33,13 @@ from .forms import MissionForm, StageUnlimitedForm, UserLineCreateForm
 # Create your views here.
 def test(request):
     user = request.user
+    stage = Stage.objects.all().first()
+    _sum = 0
+    for i in range(1, 1000):
+        coin = stage.get_prize_coin()
+        _sum += coin
+        print(coin)
+    print('===sum===' + _sum.__str__())
     # ====init
     # line = Line.objects.all().filter(line_type=2).first()
     # u_l = UserLine()
@@ -44,9 +55,9 @@ def test(request):
 
     # print(ret)
     # ==update missions
-    user_stage = UserStage.objects.get(id=1)
-    user_stage.add_user_missions()
-    return
+    # user_stage = UserStage.objects.get(id=1)
+    # user_stage.add_user_missions()
+    return HttpResponse("Ok")
 
 
 # ======Stage start =========
@@ -84,12 +95,18 @@ class StageCreate(CreateView):
     form_class = StageUnlimitedForm
     template_name = 'bee_django_mission/stage/form.html'
 
-    def form_valid(self, form):
-        stage = form.save(commit=False)
-        line = Line.objects.get(id=self.kwargs["line_id"])
-        stage.line = line
-        stage.save()
-        return super(StageCreate, self).form_valid(form)
+    # def get_context_data(self, **kwargs):
+    #     context = super(StageCreate, self).get_context_data(**kwargs)
+    #     line = Line.objects.get(id=self.kwargs["line_id"])
+    #     context["line"] = line
+    #     return context
+    #
+    # def form_valid(self, form):
+    #     stage = form.save(commit=False)
+    #     line = Line.objects.get(id=self.kwargs["line_id"])
+    #     stage.line = line
+    #     stage.save()
+    #     return super(StageCreate, self).form_valid(form)
 
 
 @method_decorator(cls_decorator(cls_name='StageUpdate'), name='dispatch')
@@ -97,6 +114,12 @@ class StageUpdate(UpdateView):
     model = Stage
     form_class = StageUnlimitedForm
     template_name = 'bee_django_mission/stage/form.html'
+    #
+    # def get_context_data(self, **kwargs):
+    #     context = super(StageUpdate, self).get_context_data(**kwargs)
+    #     line = Line.objects.get(stage__id=self.kwargs["pk"])
+    #     context["line"] = line
+    #     return context
 
 
 @method_decorator(cls_decorator(cls_name='StageDelete'), name='dispatch')
@@ -118,10 +141,10 @@ class MissionList(ListView):
     context_object_name = 'mission_list'
     paginate_by = 20
 
-    def get_context_data(self, **kwargs):
-        context = super(MissionList, self).get_context_data(**kwargs)
-
-        return context
+    # def get_context_data(self, **kwargs):
+    #     context = super(MissionList, self).get_context_data(**kwargs)
+    #
+    #     return context
 
 
 class MissionDetail(DetailView):
@@ -175,95 +198,6 @@ class MissionDelete(DeleteView):
         return self.http_method_not_allowed(request, *args, **kwargs)
 
 
-# ======Mission end =========
-
-# class CommingSoon(TemplateView):
-#     template_name = 'bee_django_mission/mission/comming_soon.html'
-
-
-
-#     template_name_week = 'bee_django_mission/user/mission/week_list.html'
-#     template_name_unlimited = 'bee_django_mission/user/mission/unlimited_list.html'
-#
-#     def get_user_stage(self):
-#         user_stage_id = self.kwargs["pk"]
-#         return get_object_or_404(UserStage, pk=user_stage_id)
-#
-#     def get_user_line(self):
-#         user_stage = self.get_user_stage()
-#         return get_object_or_404(UserLine, userstage=user_stage)
-#
-#     def get_user_stage_list(self):
-#         user_line = self.get_user_line()
-#         return user_line.get_user_all_stage()
-#
-#     def get_context_data(self, **kwargs):
-#         context = super(UserStageDetail, self).get_context_data(**kwargs)
-#         user_stage = self.get_user_stage()
-#         user_stage_list = self.get_user_stage_list()
-#         user_mission_list = UserMission.objects.filter(user_stage=user_stage)
-#         context["user_stage_list"] = user_stage_list
-#         context["user_mission_list"] = user_mission_list
-#         context["user_stage_status"] = user_stage.stutas
-#         return context
-#
-#     def get_template_names(self):
-#         user_line = self.get_user_line()
-#         line = user_line.line
-#         if line.is_unlimited_line():
-#             return self.template_name_unlimited
-#         if line.is_week_line():
-#             return self.template_name_week
-#
-#
-# class UserMissionList(RedirectView):
-#     success_pattern_name = "bee_django_mission:user_stage_detail"
-#     comming_soon_pattern_name = "bee_django_mission:comming_soon"
-#
-#     def get_line_type(self):
-#         return 0
-#
-#     def get_redirect_url(self, *args, **kwargs):
-#         # 获取周任务，没有则添加
-#
-#         user_id = self.kwargs["user_id"]
-#         user = get_object_or_404(User, pk=user_id)
-#         line_type = self.get_line_type()
-#         user_line = None
-#         user_stage = None
-#
-#         try:
-#             user_line = UserLine.objects.get(user=user, line__line_type=line_type)
-#         except:
-#             if line_type == 2:
-#                 user_line = check_add_user_line(user, line_type, check=False)
-#
-#         # 如果没有user_line,跳转错误页面
-#         if not user_line:
-#             self.url = reverse(self.comming_soon_pattern_name, kwargs={"user_id": user_id, "line_type": line_type})
-#             return super(UserMissionList, self).get_redirect_url(*args, **kwargs)
-#
-#         # 长线任务，获取进行中的或者已完成的user_stage
-#         if user_line.line.is_unlimited_line():
-#             user_stage = user_line.get_woking_user_stage()
-#             if not user_stage:
-#                 # TODO获取完成的一个user_stage
-#                 user_stage = None
-#
-#         # 周任务，获取本周user_stage
-#         elif user_line.line.is_week_line():
-#             user_stage = get_current_week_stage(user)
-#
-#
-#         if user_stage:
-#             # 更新mission状态
-#             user_stage.update_user_mission()
-#             # 更新阶段任务的完成状态
-#             user_stage.update_user_stage_status()
-#             self.url = reverse(self.success_pattern_name, kwargs={"pk": user_stage.id})
-#         return super(UserMissionList, self).get_redirect_url(*args, **kwargs)
-
-
 # 周任务
 class UserMissionListWeek(TemplateView):
     line_type = 2
@@ -277,18 +211,22 @@ class UserMissionListWeek(TemplateView):
         try:
             user_line = UserLine.objects.get(user=user, line__line_type=self.line_type)
         except:
-            user_line = check_add_user_line(user, 2, check=False)
+            user_line, msg = check_add_user_line(user, 2, check=False)
             if not user_line:
-                messages.error(self.request, "还没有周任务线")
+                messages.error(self.request, msg)
                 return context
 
 
-        # 获取[进行中/已完成/未完成/可完成但未完成]的user_stage
-        user_stage = get_current_week_stage(user)
-        # 没有本周任务，则新增加一个
-        if not user_stage:
-            user_stage = user_line.check_add_user_stage(check=False)
-
+        # # 获取[进行中/已完成/未完成/可完成但未完成]的user_stage
+        # user_stage = get_current_week_stage(user)
+        # # 没有本周任务，则新增加一个
+        # if not user_stage:
+        #     user_stage = user_line.check_add_user_stage(check=False)
+        new_user_stage = user_line.check_add_user_stage()
+        if new_user_stage:
+            user_stage = new_user_stage
+        else:
+            user_stage = get_current_week_stage(user)
         if not user_stage:
             messages.error(self.request, "还没有周任务")
             return context
@@ -303,6 +241,12 @@ class UserMissionListWeek(TemplateView):
         context["user_stage"] = user_stage
         context["user_stage_list"] = user_stage_list
         context["user_mission_list"] = user_mission_list
+        finish_rank_list = UserStageFinishCount.objects.filter(line=user_stage.user_line.line).filter(
+            finish_count__gt=0).order_by("-finish_count")[:10]
+        context["finish_rank_list"] = finish_rank_list
+        context["max_prize_coin"] = user_stage.stage.get_max_prize_coin()
+        context["prize_coin_list"] = user_stage.stage.stageprize_set.order_by("-coin")
+        context["top_prize_list"] = get_top_prize()
         return context
 
 
@@ -340,7 +284,7 @@ class UserMissionListUnlimited(TemplateView):
         context["user"] = user
         context["user_stage"] = user_stage
         context["user_stage_list"] = user_stage_list
-        context["user_mission_list"] = user_mission_list
+        # context["user_mission_list"] = user_mission_list
         return context
 
 
@@ -348,16 +292,25 @@ class UserLineCreate(CreateView):
     model = UserLine
     form_class = UserLineCreateForm
     template_name = 'bee_django_mission/user/line/form.html'
+    success_url = None
+
+    def get_context_data(self, **kwargs):
+        context = super(UserLineCreate, self).get_context_data(**kwargs)
+        user_id = self.kwargs["user_id"]
+        user = get_object_or_404(User, pk=user_id)
+        context['user'] = user
+        return context
+
     # def post(self, request, *args, **kwargs):
     def form_valid(self, form):
-        if form.is_valid():
-            user_line = form.save(commit=False)
-            user_id = self.kwargs["user_id"]
-            user = get_object_or_404(User, pk=user_id)
-            user_line.user = user
-            user_line.save()
-
-        return reverse('bee_django_mission:user_line_unlimited', kwargs={"user_id": self.kwargs["user_id"]})
+        user_line = form.save(commit=False)
+        line = user_line.line
+        user_id = self.kwargs["user_id"]
+        user = get_object_or_404(User, pk=user_id)
+        user_line.user = user
+        user_line.save()
+        self.success_url = reverse('bee_django_mission:user_line_unlimited', kwargs=self.kwargs)
+        return super(UserLineCreate, self).form_valid(form)
 
 
 @method_decorator(cls_decorator(cls_name='UserStageDetail'), name='dispatch')
@@ -368,7 +321,7 @@ class UserStageDetail(DetailView):
 
     def get_context_data(self, **kwargs):
         context = super(UserStageDetail, self).get_context_data(**kwargs)
-        user_stage = get_object_or_404(UserStage,pk=self.kwargs["pk"])
+        user_stage = get_object_or_404(UserStage, pk=self.kwargs["pk"])
         # 更新mission状态
         user_stage.update_user_mission()
         # 更新阶段任务的完成状态
@@ -381,3 +334,93 @@ class UserStageDetail(DetailView):
         context["user_stage_list"] = user_stage_list
         context["user_mission_list"] = user_mission_list
         return context
+
+
+class UserStageUpdateStatus(TemplateView):
+    def post(self, request, *args, **kwargs):
+        # This method is called when valid form data has been POSTed.
+        # It should return an HttpResponse.
+        try:
+            user_stage_id = self.kwargs['user_stage_id']
+            status = self.kwargs['status']
+            user_stage = get_object_or_404(UserStage, pk=user_stage_id)
+            # 完成任务，并发奖励
+            user_stage.finish_user_stage_add_prize()
+            # 检查添加下一阶段任务
+            user_stage.user_line.check_add_user_stage()
+            res = {"error": 0, "msg": "操作成功"}
+        except Exception as e:
+            print(e)
+            res = {"error": 1, "msg": "操作失败"}
+
+        return JSONResponse(json.dumps(res, ensure_ascii=False))
+
+
+@method_decorator(permission_required('bee_django_mission.can_reset_user_mission'), name='dispatch')
+class UserStageUpdateMission(TemplateView):
+    def post(self, request, *args, **kwargs):
+        # This method is called when valid form data has been POSTed.
+        # It should return an HttpResponse.
+        try:
+            user_stage_id = self.kwargs['user_stage_id']
+            user_stage = get_object_or_404(UserStage, pk=user_stage_id)
+            user_stage.update()
+            res = {"error": 0, "msg": "操作成功"}
+        except Exception as e:
+            print(e)
+            res = {"error": 1, "msg": "操作失败"}
+
+        return JSONResponse(json.dumps(res, ensure_ascii=False))
+
+
+class UserStageUpdate(UpdateView):
+    model = UserMission
+    form_class = UserMissionForm
+    template_name = 'bee_django_mission/user/mission/form.html'
+
+    def get_success_url(self):
+        user_mission = get_object_or_404(UserMission, id=self.kwargs["pk"])
+        user = user_mission.user_stage.user_line.user
+        line = user_mission.user_stage.user_line.line
+        if line.is_week_line():
+            return reverse('bee_django_mission:user_line_week', kwargs={"user_id": user.id})
+        if line.is_unlimited_line():
+            return reverse('bee_django_mission:user_unlimited_week', kwargs={"user_id": user.id})
+
+    @transaction.atomic
+    def form_valid(self, form):
+        if not self.request.user.has_perm('bee_django_mission.add_customusermission'):
+            messages.error(self.request, '没有权限')
+            return redirect(reverse('bee_django_mission:user_stage_update', kwargs=self.kwargs))
+
+        if form.is_valid():
+            # 新建或修改规则模版
+            user_mission = form.save(commit=True)
+            user = user_mission.user_stage.user_line.user
+            stage = user_mission.user_stage.stage
+            try:
+                custom_mission = CustomUserMission.objects.get(user=user, stage=stage,
+                                                               mission=user_mission.mission)
+            except:
+                custom_mission = CustomUserMission()
+                custom_mission.user = user
+                custom_mission.stage = stage
+                custom_mission.mission = user_mission.mission
+            custom_mission.custom_name = user_mission.custom_name
+            custom_mission.custom_count = user_mission.custom_count
+            custom_mission.save()
+            messages.success(self.request, '修改成功')
+        return super(UserStageUpdate, self).form_valid(form)
+
+
+class UserStageFinishRankList(TemplateView):
+    template_name = 'bee_django_mission/user/stage/finish/rank_list.html'
+
+    def get_context_data(self, **kwargs):
+        context = super(UserStageFinishRankList, self).get_context_data(**kwargs)
+        line = get_object_or_404(Line, pk=self.kwargs["line_id"])
+        finish_rank_list = UserStageFinishCount.objects.filter(line=line).filter(
+            finish_count__gt=0).order_by("-finish_count")
+        context["line"] = line
+        context["finish_rank_list"] = finish_rank_list
+        return context
